@@ -114,6 +114,8 @@ struct _GUPnPNetworkManagerPrivate {
         GList *nm_devices;
 
         GCancellable *cancellable;
+
+        GDBusConnection *system_bus;
 };
 
 static NMDevice *
@@ -125,7 +127,7 @@ nm_device_new (GUPnPNetworkManager *manager,
         nm_device = g_slice_new0 (NMDevice);
 
         nm_device->manager = manager;
-        nm_device->proxy = g_object_ref (device_proxy);
+        nm_device->proxy = device_proxy;
 
         return nm_device;
 }
@@ -649,6 +651,7 @@ gupnp_network_manager_constructed (GObject *object)
 
         priv->cancellable = g_cancellable_new ();
         priv->nm_devices = NULL;
+        priv->system_bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, NULL);
 
         init_network_manager (manager);
 
@@ -692,6 +695,11 @@ gupnp_network_manager_dispose (GObject *object)
                 priv->cancellable = NULL;
         }
 
+        if (priv->system_bus != NULL) {
+                g_object_unref (priv->system_bus);
+                priv->system_bus = NULL;
+        }
+
         /* Call super */
         object_class = G_OBJECT_CLASS (gupnp_network_manager_parent_class);
         object_class->dispose (object);
